import { Component, OnInit } from '@angular/core';
import { Loop } from 'src/app/models/loop';
import { FileService } from 'src/app/services/file.service';
import { ProjectService } from 'src/app/services/project.service';
import { Router, ActivatedRoute } from '@angular/router';
import { FlowService } from 'src/app/services/flow.service';
import { FormGroup, FormBuilder, FormArray, Validators } from '@angular/forms';
import ValidationUtils from 'src/app/utils/validation';

@Component({
  selector: 'app-loop',
  templateUrl: './loop.component.html',
  styleUrls: ['./loop.component.css']
})
export class LoopComponent implements OnInit {

  actualProject: number;
  actualLoop: string;
  diagram: string;
  isLoading: boolean;
  loop: Loop;
  highlightElements: Array<string>;
  public formGroup: FormGroup;

  constructor(private _fb: FormBuilder, private fileService: FileService, private flowService: FlowService, private router: Router, private route: ActivatedRoute) {
    this.route.params.subscribe(params => {
      this.actualProject = params['projectId'];
      this.actualLoop = params['loopId'];
    })
  }

  async ngOnInit() {
    await this.loadLoop();
    await this.loadBPMNDiagram();
  }

  async loadLoop() {
    this.isLoading = true;
    try {
      let result = await this.flowService.getLoop(this.actualProject, this.actualLoop);
      this.loop = result;
      this.highlight();

      this.initFormGroup();
      this.initFormObj();
    } finally {
      this.isLoading = false;
    }
  }

  async loadBPMNDiagram() {
    this.isLoading = true;
    try {
      let result = await this.fileService.getBPMNDiagram(this.actualProject);
      this.diagram = result;
    } finally {
      this.isLoading = false;
    }
  }

  highlight() {
    this.highlightElements = [];
    this.highlightElements.push(this.loop.start);
    this.highlightElements.push(this.loop.follower);
    this.highlightElements.push(this.loop.end);
  }

  initFormGroup() {
    this.formGroup = this._fb.group({
      frequencies: this._fb.array([])
    });

    this.loop.frequencies.forEach((frequency, index) => {
      this.addFrequencyToFormGroup();
    });
  }

  addFrequencyToFormGroup() {
    let formGroupFrequency = this._fb.group({
      frequency: ['', Validators.compose([Validators.required])],
    });

    this.frequenciesForm.push(formGroupFrequency);
  }

  get frequenciesForm(): FormArray {
    return <FormArray>this.formGroup.get('frequencies');
  }

  getFrequenciesForm(i: number) {
    return (<FormArray>(<FormArray>this.formGroup.get('frequencies'))
      .controls[i].get('frequency'));
  }

  initFormObj() {
    let value = {
      frequencies: []
    }

    this.loop.frequencies.forEach((frequencyVal, index) => {
      value.frequencies.push({ frequency: frequencyVal })
    });

    this.formGroup.patchValue(value);
  }

  async saveLoop() {
    if (!this.formGroup.invalid) {

      this.isLoading = true;

      this.extractFrequencies();

      try {
        let result = await this.flowService.updateLoop(this.actualProject, this.loop);
        this.router.navigate(['../'], { relativeTo: this.route });
      } finally {
        this.isLoading = false;
      }
    } else {
      ValidationUtils.validateAllFormFields(this.formGroup);
    }
  }

  extractFrequencies() {
    const formValue = this.formGroup.value;
    this.loop.frequencies = [];

    formValue.frequencies
      .map((element) => element.frequency)
      .filter((value, index, array) => array.indexOf(value) === index) //distinct
      .forEach((f) => this.loop.frequencies.push(f));
  }

  async deleteFrequency(index: number) {
    this.frequenciesForm.removeAt(index);
    this.loop.frequencies.splice(index, 1);
  }

  async addFrequency() {
    this.addFrequencyToFormGroup();
    this.loop.frequencies.push(null);
  }
}
